local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local MinimapModule = idTweaks:NewModule('Minimap', 'AceConsole-3.0')

local TL, TC, TR = 'TOPLEFT', 'TOP', 'TOPRIGHT'
local ML, MC, MR = 'LEFT', 'CENTER', 'RIGHT'
local BL, BC, BR = 'BOTTOMLEFT', 'BOTTOM', 'BOTTOMRIGHT'

MinimapModule.DEBUG = false

function MinimapModule:Debug(...)
  if MinimapModule.DEBUG then
    self:Print(...)
  end
end

function MinimapModule:setDefaults()
  self:Debug('MinimapModule:setDefaults')

  self.defaults = {
    profile = {
      shape = {
        square = true,
        position = true,
      },
      hide = {
        areaText = true,
        calendarButton = true,
        northTag = true,
        timeDisplay = true,
        worldMapButton = true,
        zoomButtons = true,
      },
      zoom = {
        mouseWheel = true
      },
    }
  }
end

function MinimapModule:setDatabase()
  self:Debug('MinimapModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('Minimap', self.defaults)
end

function MinimapModule:setOptions()
  self:Debug('MinimapModule:setOptions')

  self.options = {
    name = 'Minimap',
    desc = 'Changes how the Minimap works.',
    handler = self,
    type = 'group',
    args = {
      shape = {
        order = 1,
        name = 'Shape',
        desc = 'Shape settings.',
        handler = self,
        type = 'group',
        args = {
          square = {
            order = 1,
            name = 'Square',
            desc = 'Makes the minimap square',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.shape.square
            end,
            set = function(_, value)
              self.db.profile.shape.square = value
              self.shape.square:check()
            end
          },
          position = {
            order = 1,
            name = 'Position',
            desc = 'Move the minimap next to the action bars',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.shape.position
            end,
            set = function(_, value)
              self.db.profile.shape.position = value
              self.shape.position:check()
            end
          },
        },
      },

      zoom = {
        order = 2,
        name = 'Zoom',
        desc = 'Change how zooming the Minimap works.',
        handler = self,
        type = 'group',
        args = {
          mouseWheel = {
            order = 1,
            name = 'Mouse Wheel',
            desc = 'Enables Zooming the minimap with the Mouse Wheel.',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.zoom.mouseWheel
            end,
            set = function(_, value)
              self.db.profile.zoom.mouseWheel = value
              self.zoom:checkMousewheelZoomingEnabled()
            end
          }
        }
      },

      hide = {
        order = 3,
        name = 'Hide',
        desc = 'Toggle visibility of parts of the Minimap with these options.',
        handler = self,
        type = 'group',
        args = {
          areaText = {
            name = 'Area Text',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.hide.areaText
            end,
            set = function(_, value)
              self.db.profile.hide.areaText = value
              self.hide:checkFrame('areaText')
            end
          },

          calendarButton = {
            name = 'Calendar Button',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.hide.calendarButton
            end,
            set = function(_, value)
              self.db.profile.hide.calendarButton = value
              self.hide:checkFrame('calendarButton')
            end
          },

          northTag = {
            name = 'North Tag',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.hide.northTag
            end,
            set = function(_, value)
              self.db.profile.hide.northTag = value
              self.hide:checkFrame('northTag')
            end
          },

          timeDisplay = {
            name = 'Time Display',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.hide.timeDisplay
            end,
            set = function(_, value)
              self.db.profile.hide.timeDisplay = value
              self.hide:checkFrame('timeDisplay')
            end
          },

          worldMapButton = {
            name = 'World Map Button',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.hide.worldMapButton
            end,
            set = function(_, value)
              self.db.profile.hide.worldMapButton = value
              self.hide:checkFrame('worldMapButton')
            end
          },

          zoomButtons = {
            name = 'Zoom Buttons',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.hide.zoomButtons
            end,
            set = function(_, value)
              self.db.profile.hide.zoomButtons = value
              self.hide:checkFrame('zoomButtons')
            end
          },
        }
      }
    }
  }
end

function MinimapModule:registerOptions()
  self:Debug('MinimapModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('idTweaks-Minimap', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('idTweaks-Minimap', 'Minimap', 'idTweaks')
end

function MinimapModule:OnInitialize()
  self:Debug('MinimapModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function MinimapModule:OnEnable()
  self:Debug('MinimapModule:OnEnable')

  self:registerOptions()

  self.shape.square:check()
  self.shape.position:check()
  self.zoom:checkEnabled()
  self.hide:checkEnabled()
end

function MinimapModule:OnDisable()
  self:Debug('MinimapModule:OnDisable')

  self.shape.square:disable()
  self.shape.position:disable()
  self.zoom:disable()
  self.hide:disable()
end

MinimapModule.shape = {
  square = {
    check = function()
      if MinimapModule.db.profile.shape.square == true then
        MinimapModule.shape.square:enable()
      else
        MinimapModule.shape.square:disable()
      end
    end,

    enable = function()
      Minimap:SetMaskTexture('Interface\\Buttons\\WHITE8x8')
      MinimapBorder:Hide()
    end,

    disable = function()
      Minimap:SetMaskTexture('textures\\MinimapMask')
      MinimapBorder:Show()
    end,
  },

  position = {
    check = function()
      if MinimapModule.db.profile.shape.position == true then
        MinimapModule.shape.position:enable()
      else
        MinimapModule.shape.position:disable()
      end
    end,

    enable = function()
      Minimap:ClearAllPoints()
      Minimap:SetParent(UIParent)
      Minimap:SetPoint(BL, idTweaksActionBar1, BR, 0, 0)
      Minimap:SetWidth(idTweaksActionBar1:GetHeight() * 5)
      Minimap:SetHeight(idTweaksActionBar1:GetHeight() * 5)
    end,

    disable = function()
      Minimap:ClearAllPoints()
      Minimap:SetParent(MinimapCluster)
      Minimap:SetPoint(MC, MinimapCluster, TC, 9, -92)
      Minimap:SetWidth(140)
      Minimap:SetHeight(140)
    end,
  },
}

function GetMinimapShape()
  return MinimapModule.db.profile.shape.square == true and 'SQUARE' or 'ROUND'
end

MinimapModule.zoom = {}

function MinimapModule.zoom:disable()
  self:Debug('MinimapModule.zoom:disable')

  MinimapModule.zoom:disableMouseWheelZooming()
end

function MinimapModule.zoom:checkEnabled()
  MinimapModule.zoom:checkMousewheelZoomingEnabled()
end

function MinimapModule.zoom:checkMousewheelZoomingEnabled()
  MinimapModule:Debug('MinimapModule.zoom:checkMousewheelZoomingEnabled')

  if MinimapModule.db.profile.zoom.mouseWheel == true then
    MinimapModule.zoom:enableMouseWheelZooming()
  else
    MinimapModule.zoom:disableMouseWheelZooming()
  end
end

function MinimapModule.zoom:enableMouseWheelZooming()
  MinimapModule:Debug('MinimapModule.zoom:enableMouseWheelZooming')

  Minimap:EnableMouseWheel(true)
  Minimap:SetScript('OnMouseWheel', MinimapModule.zoom.handleMousewheelZoom)
end

function MinimapModule.zoom:disableMouseWheelZooming()
  MinimapModule:Debug('MinimapModule.zoom:disableMouseWheelZooming')

  Minimap:EnableMouseWheel(false)
  Minimap:SetScript('OnMouseWheel', nil)
end

function MinimapModule.zoom.handleMousewheelZoom(frame, delta)
  MinimapModule:Debug('MinimapModule.zoom.handleMousewheelZoom')

  if delta > 0 and Minimap:GetZoom() < 5 then
    Minimap:SetZoom(Minimap:GetZoom() + 1)
  elseif delta < 0 and Minimap:GetZoom() > 0 then
    Minimap:SetZoom(Minimap:GetZoom() - 1)
  end
end

MinimapModule.hide = {
  frames = {
    -- MinimapTrackingFrame
    -- MinimapMailFrame
    -- MinimapBattlefieldFrame

    areaText = {
      enable = function()
        MinimapBorderTop:Hide()
        MinimapZoneText:Hide()
        MinimapZoneTextButton:Hide()
        MinimapToggleButton:Hide()
      end,
      disable = function()
        MinimapBorderTop:Show()
        MinimapZoneText:Show()
        MinimapZoneTextButton:Show()
        MinimapToggleButton:Show()
      end,
    },

    calendarButton = {
      enable = function()
        GameTimeFrame:Hide()
      end,
      disable = function()
        GameTimeFrame:Show()
      end,
    },

    northTag = {
      enable = function()
        MinimapNorthTag:SetAlpha(0)
      end,
      disable = function()
        MinimapNorthTag:SetAlpha(1)
      end,
    },

    timeDisplay = {
      enable = function()
        print('timeDisplay:enable')
        TimeManagerClockButton:Hide()
      end,
      disable = function()
        print('timeDisplay:disable')
        TimeManagerClockButton:Show()
      end,
    },

    worldMapButton = {
      enable = function()
        MiniMapWorldMapButton:Hide()
      end,
      disable = function()
        MiniMapWorldMapButton:Show()
      end,
    },

    zoomButtons = {
      enable = function()
        MinimapZoomIn:Hide()
        MinimapZoomOut:Hide()
      end,

      disable = function()
        MinimapZoomIn:Show()
        MinimapZoomOut:Show()
      end
    }
  }
}

function MinimapModule.hide:disable()
  MinimapModule:Debug('MinimapModule.hide:disable')

  for name, functions in pairs(self.frames) do
    self.frames[name].disable()
  end
end

function MinimapModule.hide:checkEnabled()
  MinimapModule:Debug('MinimapModule.hide:checkEnabled')

  for name, functions in pairs(self.frames) do
    self:checkFrame(name)
  end
end

function MinimapModule.hide:checkFrame(name)
  MinimapModule:Debug('MinimapModule.hide:checkFrame')

  if MinimapModule.db.profile.hide[name] == true then
    self.frames[name].enable()
  else
    self.frames[name].disable()
  end
end

idTweaks.MinimapModule = MinimapModule
